 /*
 Game for CPC464 The Great Battles of Magistar, based on North & South.>
   Copyright (C) 2016 Shirkamdev Xnetrix 

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
   */
#include "game_system_methods.h"

u8 comprobarPuntuaciones() {
	TGenericUnit* start;

	start = allyUnits;
	if(unidadesPuntuando(start)) {
		drawPuntosAlly();
	}

	start = enemyUnits;
	if(unidadesPuntuando(start)) {
		drawPuntosEnemy();
	}
	//Si no quedan unidades, quiere decir que ya se ha acabado el juego
	return !quedanUnidades();
}


u8 unidadesPuntuando(TGenericUnit* start) __z88dk_fastcall {
	TGenericUnit* end = start + MAX_UNITS;

	while(start!=end) {
		//If the unit is in the border and is not fainted
		if((start->x == LIMITE_DERECHA || start->x == 1)&& start->lifeUnit != 0) {
			return 1; //return true
		}
		++start;
	}

	return 0;
}



void drawPuntosAlly() {
	u8 points=0;
	TGenericUnit* start = allyUnits;
	TGenericUnit* end = allyUnits+MAX_UNITS;

	while(start!=end) {
		if(start->x == LIMITE_DERECHA && start->lifeUnit != 0) {
			points = start->lifeUnit/32;
			puntosAlly+=points;
			while(points != 0) {
				drawPoint(&currentAllyPointDraw);
				currentAllyPointDraw.x++;
				if(currentAllyPointDraw.x > 35) {
					currentAllyPointDraw.y+=2;
					currentAllyPointDraw.x = 33;
				}
				--points;
			}
			start->lifeUnit=0;
		}
		++start;
	}
}

void drawPuntosEnemy() {
	u8 points=0;
	TGenericUnit* start = enemyUnits;
	TGenericUnit* end = enemyUnits+MAX_UNITS;

	while(start!=end) {
		if(start->x == 1 && start->lifeUnit != 0) {
			points = start->lifeUnit/32;
			puntosEnemy+=points;
			while(points != 0) {
				drawPoint(&currentEnemyPointDraw);
				currentEnemyPointDraw.x++;
				if(currentEnemyPointDraw.x >= 46) {
					currentEnemyPointDraw.y+=2;
					currentEnemyPointDraw.x = 43;
				}
				--points;
			}
			start->lifeUnit=0;
		}
		++start;
	}
}

u8 quedanUnidades() {
	u8 unidades = 0;
	u8* startLeft = leftAllyUnits;
	u8* endLeft = startLeft+3;
	TGenericUnit* start = allyUnits;
	TGenericUnit* end = start+MAX_UNITS;

	//Comprobamos si quedan unidades aliadas por poner
	while(startLeft != endLeft) {
		unidades += *startLeft;
		++startLeft;
	}

	//Si no quedan unidades aliadas por poner, comprobamos si le quedan al enemigo
	if(unidades == 0) {
		startLeft = leftEnemyUnits;
		endLeft = startLeft+3;

		while(startLeft != endLeft) {
			unidades += *startLeft;
			++startLeft;
		}

		//Si siguen sin quedar unidades, hemos de comprobar que no quedan
		//en el propio mapa, o sea, que se estan moviendo
		if(unidades == 0) {
			while(start!=end) {
				if(!unidadMuerta(start)) {
					++unidades;
				}
				++start;
			}

			//Ya comprobamos si al enemigo le quedan unidades
			if(unidades == 0) {
				start = enemyUnits;
				end = start+MAX_UNITS;

				while(start!=end) {
					if(!unidadMuerta(start)) {
						++unidades;
					}
					++start;
				}
			}
		}
	}

	return unidades;
}

void initializeGame(u8 mapSelected) {
	switch(mapSelected) {
		case 1:
			leftAllyUnits[0] = 1;
			leftAllyUnits[1] = 2;
			leftAllyUnits[2] = 1;
			selectedAllyUnitType = 0;

			leftEnemyUnits[0] = 1;
			leftEnemyUnits[1] = 2;
			leftEnemyUnits[2] = 1;

			drawMap(mapSelected);
			break;
		case 2:
			leftAllyUnits[0] = 3;
			leftAllyUnits[1] = 2;
			leftAllyUnits[2] = 1;
			selectedAllyUnitType = 0;

			leftEnemyUnits[0] = 2;
			leftEnemyUnits[1] = 1;
			leftEnemyUnits[2] = 3;

			drawMap(mapSelected);
			break;
		case 3:
			leftAllyUnits[0] = 2;
			leftAllyUnits[1] = 2;
			leftAllyUnits[2] = 2;
			selectedAllyUnitType = 0;

			leftEnemyUnits[0] = 1;
			leftEnemyUnits[1] = 3;
			leftEnemyUnits[2] = 2;

			drawMap(mapSelected);
			break;

		default:
			leftAllyUnits[0] = 1;
			leftAllyUnits[1] = 2;
			leftAllyUnits[2] = 1;
			selectedAllyUnitType = 0;

			leftEnemyUnits[0] = 1;
			leftEnemyUnits[1] = 2;
			leftEnemyUnits[2] = 1;

			drawMap(1);
	}
	
	drawBanner();
	drawCursor();
}

void teRindes() {
	cpct_scanKeyboard_f();

	if(cpct_isAnyKeyPressed()) {
		if(cpct_isKeyPressed(Key_Q)) {
			rendirse = 1;
		}
	}
}

void comprobarWinner() {
	if(puntosAlly > puntosEnemy) {
		drawWinner(1);
	}
	else if(puntosAlly < puntosEnemy){
		drawWinner(2);
	}
	else {
		drawWinner(3);
	}

	wait();
}

void wait() {
	do {
		cpct_scanKeyboard_f();
	}while(!cpct_isKeyPressed(Key_Q));
}
